@@ -10,12 +10,14 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkConsole.c,v 1.3 1999/02/04 20:51:36 stanton Exp $
+ * RCS: @(#) $Id: tkConsole.c,v 1.6 1999/04/28 18:18:06 redman Exp $
  */
 
 #include "tk.h"
 #include <string.h>
 
+#include "tkInt.h"
+
 /*
  * A data structure of the following type holds information for each console
  * which a handler (i.e. a Tcl command) has been defined for a particular
@@ -27,7 +29,17 @@ typedef struct ConsoleInfo {
     Tcl_Interp *interp;		/* Interpreter to send console commands. */
 } ConsoleInfo;
 
-static Tcl_Interp *gStdoutInterp = NULL;
+typedef struct ThreadSpecificData {
+    Tcl_Interp *gStdoutInterp;
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
+static int consoleInitialized = 0;
+
+/* 
+ * The Mutex below is used to lock access to the consoleIntialized flag
+ */
+
+TCL_DECLARE_MUTEX(consoleMutex)
 
 /*
  * Forward declarations for procedures defined later in this file:
@@ -35,8 +47,6 @@ static Tcl_Interp *gStdoutInterp = NULL;
  * The first three will be used in the tk app shells...
  */
  
-void	TkConsoleCreate _ANSI_ARGS_((void));
-int	TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
 void	TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
 			    int devId, char *buffer, long size));
 
@@ -79,7 +89,7 @@ static Tcl_ChannelType consoleChannelType = {
 /*
  *----------------------------------------------------------------------
  *
- * TkConsoleCreate --
+ * Tk_InitConsoleChannels --
  *
  * 	Create the console channels and install them as the standard
  * 	channels.  All I/O will be discarded until TkConsoleInit is
@@ -96,37 +106,86 @@ static Tcl_ChannelType consoleChannelType = {
  */
 
 void
-TkConsoleCreate()
+Tk_InitConsoleChannels(interp)
+    Tcl_Interp *interp;
 {
     Tcl_Channel consoleChannel;
 
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console0",
-	    (ClientData) TCL_STDIN, TCL_READABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
+    /*
+     * Ensure that we are getting the matching version of Tcl.  This is
+     * really only an issue when Tk is loaded dynamically.
+     */
+
+    if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
+        return;
     }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDIN);
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console1",
-	    (ClientData) TCL_STDOUT, TCL_WRITABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
+
+    Tcl_MutexLock(&consoleMutex);
+    if (!consoleInitialized) {
+
+	consoleInitialized = 1;
+	
+	/*
+	 * check for STDIN, otherwise create it
+	 */
+
+	if (Tcl_GetStdChannel(TCL_STDIN) == NULL) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console0",
+		    (ClientData) TCL_STDIN, TCL_READABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDIN);
+	}
+
+	/*
+	 * check for STDOUT, otherwise create it
+	 */
+	
+	if (Tcl_GetStdChannel(TCL_STDOUT) == NULL) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console1",
+		    (ClientData) TCL_STDOUT, TCL_WRITABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDOUT);
+	}
+	
+	/*
+	 * check for STDERR, otherwise create it
+	 */
+	
+	if (Tcl_GetStdChannel(TCL_STDERR) == NULL) {
+	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console2",
+		    (ClientData) TCL_STDERR, TCL_WRITABLE);
+	    if (consoleChannel != NULL) {
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-translation", "lf");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-buffering", "none");
+		Tcl_SetChannelOption(NULL, consoleChannel,
+			"-encoding", "utf-8");
+	    }
+	    Tcl_SetStdChannel(consoleChannel, TCL_STDERR);
+	}
     }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDOUT);
-    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console2",
-	    (ClientData) TCL_STDERR, TCL_WRITABLE);
-    if (consoleChannel != NULL) {
-	Tcl_SetChannelOption(NULL, consoleChannel, "-translation", "lf");
-	Tcl_SetChannelOption(NULL, consoleChannel, "-buffering", "none");
-    }
-    Tcl_SetStdChannel(consoleChannel, TCL_STDERR);
+    Tcl_MutexUnlock(&consoleMutex);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * TkConsoleInit --
+ * Tk_CreateConsoleWindow --
  *
  *	Initialize the console.  This code actually creates a new
  *	application and associated interpreter.  This effectivly hides
@@ -142,12 +201,14 @@ TkConsoleCreate()
  */
 
 int 
-TkConsoleInit(interp)
+Tk_CreateConsoleWindow(interp)
     Tcl_Interp *interp;			/* Interpreter to use for prompting. */
 {
     Tcl_Interp *consoleInterp;
     ConsoleInfo *info;
     Tk_Window mainWindow = Tk_MainWindow(interp);
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 #ifdef MAC_TCL
     static char initCmd[] = "source -rsrc {Console}";
 #else
@@ -169,7 +230,7 @@ TkConsoleInit(interp)
     if (Tk_Init(consoleInterp) != TCL_OK) {
 	goto error;
     }
-    gStdoutInterp = interp;
+    tsdPtr->gStdoutInterp = interp;
     
     /* 
      * Add console commands to the interp 
@@ -225,11 +286,15 @@ ConsoleOutput(instanceData, buf, toWrite, errorCode)
     int toWrite;			/* How many bytes to write? */
     int *errorCode;			/* Where to store error code. */
 {
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
     *errorCode = 0;
     Tcl_SetErrno(0);
 
-    if (gStdoutInterp != NULL) {
-	TkConsolePrint(gStdoutInterp, (int) instanceData, buf, toWrite);
+    if (tsdPtr->gStdoutInterp != NULL) {
+	TkConsolePrint(tsdPtr->gStdoutInterp, (int) instanceData, buf, 
+                toWrite);
     }
     
     return toWrite;
@@ -376,6 +441,7 @@ ConsoleCmd(clientData, interp, argc, argv)
     int length;
     int result;
     Tcl_Interp *consoleInterp;
+    Tcl_DString dString;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -388,20 +454,20 @@ ConsoleCmd(clientData, interp, argc, argv)
     result = TCL_OK;
     consoleInterp = info->consoleInterp;
     Tcl_Preserve((ClientData) consoleInterp);
+    Tcl_DStringInit(&dString);
+
     if ((c == 't') && (strncmp(argv[1], "title", length)) == 0) {
-	Tcl_DString dString;
-	
-	Tcl_DStringInit(&dString);
 	Tcl_DStringAppend(&dString, "wm title . ", -1);
 	if (argc == 3) {
 	    Tcl_DStringAppendElement(&dString, argv[2]);
 	}
 	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
-	Tcl_DStringFree(&dString);
     } else if ((c == 'h') && (strncmp(argv[1], "hide", length)) == 0) {
-	Tcl_Eval(consoleInterp, "wm withdraw .");
+	Tcl_DStringAppend(&dString, "wm withdraw . ", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
     } else if ((c == 's') && (strncmp(argv[1], "show", length)) == 0) {
-	Tcl_Eval(consoleInterp, "wm deiconify .");
+	Tcl_DStringAppend(&dString, "wm deiconify . ", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
     } else if ((c == 'e') && (strncmp(argv[1], "eval", length)) == 0) {
 	if (argc == 3) {
 	    result = Tcl_Eval(consoleInterp, argv[2]);
@@ -418,6 +484,7 @@ ConsoleCmd(clientData, interp, argc, argv)
 		(char *) NULL);
         result = TCL_ERROR;
     }
+    Tcl_DStringFree(&dString);
     Tcl_Release((ClientData) consoleInterp);
     return result;
 }
@@ -533,9 +600,13 @@ ConsoleEventProc(clientData, eventPtr)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     Tcl_Interp *consoleInterp;
+    Tcl_DString dString;
     
     if (eventPtr->type == DestroyNotify) {
-        consoleInterp = info->consoleInterp;
+
+	Tcl_DStringInit(&dString);
+  
+	consoleInterp = info->consoleInterp;
 
         /*
          * It is possible that the console interpreter itself has
@@ -548,7 +619,9 @@ ConsoleEventProc(clientData, eventPtr)
             return;
         }
         Tcl_Preserve((ClientData) consoleInterp);
-	Tcl_Eval(consoleInterp, "tkConsoleExit");
+	Tcl_DStringAppend(&dString, "tkConsoleExit", -1);
+	Tcl_Eval(consoleInterp, Tcl_DStringValue(&dString));
+	Tcl_DStringFree(&dString);
         Tcl_Release((ClientData) consoleInterp);
     }
 }
